
#ifndef COMPONENTPROXY_H
#define COMPONENTPROXY_H

#include <kore/kore.h>

namespace kore
{
    class KORE_API Interface;
    class KORE_API MetaInfo;
    class ComponentProxyPrivate;

    /**
     * Korelib internal helper class.
     * Mainly used for lazy (on-demand) instantiation of components
     * and reference counting (to bo implemented in future versions).
     * Don't use it unless you have a very good reason. Use ComponentRef instead.
     */
    class KORE_API ComponentProxy
    {
    public:
        /**
         * Default ctor.
         * Creates an empty ComponentProxy. Not really usefull.
         */
        ComponentProxy();
        /**
         * Creates a ComponentProxy and sets the corresponding component meta information.
         * @param info - the component meta information.
         */
        ComponentProxy( MetaInfo* info );
        /**
         * Creates a ComponentProxy and sets the corresponding component instance 
         * and meta information.
         * @param info - the component meta information.
         * @param inst - the component instance.
         */
        ComponentProxy( MetaInfo* info, Interface* inst );
        /**
         * Dtor. Frees the corresonding component instance (if any) and meta information.
         */
        virtual ~ComponentProxy();

        /**
         * Returns the component instance.
         * It also tries to load and instantiate the component if the instance is NULL.
         * @return the component instance or NULL if a instance could not be created.
         */
        virtual Interface* instance();
        /**
         * Creates a new component instance.
         * It also tries to load the component plugin (library) if it wasn't loaded yet.
         * @return the newly created component instance or NULL if a instance could not be created.
         */
        virtual Interface* newInstance();
        /**
         * Returns the component meta information, if any.
         * @return the component meta information or NULL if there is no metainfo
         */
        virtual MetaInfo* metaInfo();
        /**
         * Sets the component instance.
         * Use this function only if you really know what you're doing.
         */
        virtual void setInstance( Interface* inst );
    private:
        ComponentProxyPrivate* _d;
    };
};

#endif
