
#ifndef LIBRARYLOADER_H
#define LIBRARYLOADER_H

#include <kore/kore.h>
#include <kore/interface.h>

namespace kore
{

    class KORE_API Library;
    class LibraryLoaderPrivate;

    /**
     * Loads plugins - C/C++ libraries and Python modules (if python support is enabled).
     * Usually you don't need to use this component directly. Use the LibraryLocator instead
     * and let korelib automagically load the plugins for you as needed.
     * Services provided by this component:
     * <b>/kore/service/loader</b>
     * /kore/service/interface
     * /kore/service/none
     */
    class KORE_API LibraryLoader: public Interface
    {
    public:
        /**
         * Plugin loading and resolving flags:
         * @param LAZY - resolve the plugin symbols as needed
         * @param NOW - resolve the plugin symbols as soon as it gets loaded
         * @param GLOBAL - make the plugin symbols global
         */
        enum { LAZY = 1, NOW, GLOBAL = 4};
        /**
         * Default ctor. Creates a LibraryLoader instance.
         * Don't use the constructor to get a LibraryLoader instance.
         * Querry the ComponentTrader instead and look for the "/kore/service/loader"
         * service.
         */
        LibraryLoader();
        /**
         * Dtor. Deletes the LibraryLoader instance.
         * Don't use the dtor unless you really know what you're doing.
         * Let korelib do the management instead.
         */
        virtual ~LibraryLoader();
        /**
         * Loads a plugin (library) and returns the corresponding Library instance.
         * Usually you don't need to use this function directly. Use the LibraryLocator instead
         * and let korelib automagically load the plugins for you as needed.
         * @param libName - the library name (not including platform-specific prefix and extension)
         * @param libPath - where to load the library from. The default library search path is used
         * if this parameter is NULL.
         * @param libFlags - library loading and resolving flags. Must be at least one of the LAZY or NOW
         * for Linux systems.
         * @return the newly loaded library or NULL of the library coun't get loaded for some reason.
         * In the latter case use lastError() to get a hint on what could have gone wrong.
         */
        virtual Library* loadLibrary(const char* libName, const char* libPath = 0, int libFlags = 0);
        /**
         *
         */
        virtual const char* lastError();

        /**
         *
         */
        virtual char* libName2fileName(const char* libName);
        /**
         *
         */
        virtual char* fileName2libName(const char* fileName);
    private:
        LibraryLoaderPrivate* _d;
    };

};

#endif
